const Gtk = imports.gi.Gtk;
const Lang = imports.lang;

const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Lib = Extension.imports.lib;
const Shared = Extension.imports.sharedConstants;

const Gettext = imports.gettext.domain('todotxt');
const _ = Gettext.gettext;

const schema = "org.gnome.shell.extensions.TodoTxt";

function init() {

}

function buildPrefsWidget() {
    let prefs = new Prefs(schema);

    return prefs.buildPrefsWidget();
}

function Prefs(schema) {
    this.init(schema);
}

Prefs.prototype = {
    settings: null,

    vboxList: null,
    hboxsList: new Array(),

    init: function(schema) {
        let settings = new Lib.Settings(schema);
        this.settings = settings.getSettings();
    },

    launchFileChooser: function(caller, entryTarget, title) {
        let dialogTitle = _("Select file");
        if (title != null) {
            dialogTitle = title;
        }
        let chooser = new Gtk.FileChooserDialog({
            title: dialogTitle,
            action: Gtk.FileChooserAction.OPEN,
            modal: true
        });

        chooser.add_button(Gtk.STOCK_CANCEL, 0);
        chooser.add_button(Gtk.STOCK_OPEN, 1);
        chooser.set_default_response(1);
        let filename = null;
        if (chooser.run() == 1) {
            filename = chooser.get_filename();
            if (filename) {
                log(filename);
                entryTarget.set_text(filename);
            }
        }
        chooser.destroy();
    },

    buildLocationSettings: function(parentContainer, text, setting, description) {
        let locationHbox = new Gtk.Box({
            orientation: Gtk.Orientation.HORIZONTAL
        });
        let locationLabel = new Gtk.Label({
            label: text,
            xalign: 0
        });
        var locationValue = new Gtk.Entry({
            hexpand: true
        });
        locationValue.set_text(this.settings.get_string(setting));
        let locationBrowse = new Gtk.Button({
            label: _("Browse")
        });
        locationBrowse.connect("clicked", Lang.bind(this, this.launchFileChooser, locationValue, description));
        let locationApply = new Gtk.Button({
            label: _("Apply")
        });
        let _this = this;
        locationApply.connect("clicked", Lang.bind(this, function(object) {
            log("Setting " + setting + " to " + locationValue.get_text());
            _this.settings.set_string(setting, locationValue.get_text());
        }));
        locationHbox.pack_start(locationLabel, true, true, 0);
        locationHbox.add(locationValue);
        locationHbox.add(locationBrowse);
        locationHbox.add(locationApply);

        parentContainer.add(locationHbox);
    },

    buildFilesSettings: function(parentContainer) {
        let label = new Gtk.Label({
            label: _("<b>Files</b>"),
            use_markup: true,
            xalign: 0
        });
        let vbox = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            margin_left: 20
        });

        this.buildLocationSettings(vbox, _("Todo.txt location"), "todotxt-location", _("Select location of todo.txt file"));
        this.buildLocationSettings(vbox, _("Done.txt location"), "donetxt-location", _("Select location of done.txt file"));

        parentContainer.add(label);
        parentContainer.add(vbox);
    },

    buildSettingsSwitch: function(parentContainer, labelText, setting) {
        let switchHbox = new Gtk.Box({
            orientation: Gtk.Orientation.HORIZONTAL
        });
        let switchLabel = new Gtk.Label({
            label: labelText,
            xalign: 0
        });
        let settingsSwitch = new Gtk.Switch({
            active: this.settings.get_boolean(setting)
        });
        let _this = this;
        settingsSwitch.connect('notify::active', Lang.bind(this, function(object) {
            log("changing boolean " + setting + " to " + object.active);
            _this.settings.set_boolean(setting, object.active);
        }));
        switchHbox.pack_start(switchLabel, true, true, 0);
        switchHbox.add(settingsSwitch);
        parentContainer.add(switchHbox);
    },

    buildComboBox: function(parentContainer, labelText, setting, options) {
        if(options == null) {
        return;
        }
        let comboHbox = new Gtk.Box({
            orientation: Gtk.Orientation.HORIZONTAL
        });
        let comboLabel = new Gtk.Label({
label: labelText,
xalign:0});
        let comboBox = new Gtk.ComboBoxText();
        if (options.length == 0){
            comboBox.sensitive = false;
        }
        for(let i = 0; i < options.length; i++) {
            comboBox.insert_text(i, options[i]);
            if(i === this.settings.get_int(setting)) {
                comboBox.set_active(i);
            }
        }
        let _this = this;
        comboBox.connect('changed', Lang.bind(this, function(object){
            log("changing " + setting + " to " + object.get_active() + " (" + options[object.get_active()] + ")");
            _this.settings.set_int(setting, object.get_active());
        }));
        comboHbox.pack_start(comboLabel, true, true, 0);
        comboHbox.add(comboBox);
        parentContainer.add(comboHbox);
    },

    buildGeneralSettings: function(parentContainer) {
        let label = new Gtk.Label({
            label: _("<b>General</b>"),
            use_markup: true,
            xalign: 0
        });
        let vbox = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            margin_left: 20
        });

        this.buildSettingsSwitch(vbox, _("Auto-archive done tasks"), "auto-archive");
        this.buildSettingsSwitch(vbox, _("Debug mode"), "debug-mode" );

        parentContainer.add(label);
        parentContainer.add(vbox);
    },

    buildGroupingSettings: function(parentContainer) {
        let label = new Gtk.Label({
            label: _("<b>Grouping</b>"),
            use_markup: true,
            xalign: 0
        });
        let vbox = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            margin_left: 20
        });

        let options = new Array();
        options[Shared.NO_GROUPING] = _("No grouping");
        options[Shared.GROUP_BY_PROJECTS] = _("Projects");
        options[Shared.GROUP_BY_CONTEXTS] = _("Contexts");
        this.buildComboBox(vbox, _("Group tasks by"), "group-by", options);
        this.buildSettingsSwitch(vbox, _("Put ungrouped tasks in seperate group"), "group-ungrouped");

        parentContainer.add(label);
        parentContainer.add(vbox);
    },

    buildPrefsWidget: function() {
        let frame = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            border_width: 10
        });

        this.buildFilesSettings(frame);
        this.buildGeneralSettings(frame);
        this.buildGroupingSettings(frame);

        frame.show_all();

        return frame;
    }
}

/* vi: set expandtab tabstop=4 shiftwidth=4: */
